#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 25 14:33:25 2019

@author: gc4217
"""

import numpy as np

def repeat_masses(Masses, n_atom_conventional_cell, n_atom_primitive_cell, N1, N2, N3):
    repeated_masses = np.array([])
    repeated_masses_for_ani = np.array([])
    for i in range(len(Masses)):
        mass = Masses[i]

        n = n_atom_conventional_cell[i]
        nprim = n_atom_primitive_cell[i]
        
        m = np.repeat(mass, N1*N2*N3*3*n)
        m_ani = np.repeat(mass,nprim*3)
        
        repeated_masses = np.concatenate((repeated_masses,m))
        repeated_masses_for_ani = np.concatenate((repeated_masses_for_ani,m_ani))
        
    masses = np.array(repeated_masses).flatten()
    masses_for_animation = np.array(repeated_masses_for_ani).flatten()
    
    return masses, masses_for_animation


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Nov 13 18:35:54 2018

@author: Gabriele Coiana
"""
import numpy as np


def getk_fcc(a,N1,N2,N3,kinput_scaled):
    # =============================================================================
    #   Brilloin zone for FCC
    c = 2*np.pi/a
    b1 = c*np.array([-1,1,1])#/np.sqrt(3)
    b2 = c*np.array([1,-1,1])#/np.sqrt(3)
    b3 = c*np.array([1,1,-1])#/np.sqrt(3)
    b = np.linalg.norm(b1)
    kinput_real = np.array(kinput_scaled)*b
    #Compute all possible k points
    x1 = np.arange(0,N1+1)/N1
    x2 = np.arange(0,N2+1)/N2
    x3 = np.arange(0,N3+1)/N3
    comb = np.array(np.meshgrid(x1,x2,x3)).T.reshape(-1,3)
    #a1 = np.multiply(comb,b1)
    #a2 = np.multiply(comb,b2)
    #a3 = np.multiply(comb,b3)
    #kpoints = a1+a2+a3
    allkpoints_scaled = comb
    allkpoints = comb*b
    # =============================================================================
    
    
    # =============================================================================
    # Compute directions <x y z>
    directions_scaled = []
    directions = []
    for i in range(0,len(kinput_scaled)-1):
        if(kinput_scaled[i] not in allkpoints_scaled):
            print('The k point '+str(kinput_scaled[i])+' provided is not compatible with the supercell. \nTry again :)')
        num = 1/np.max(np.abs(kinput_scaled[i]-kinput_scaled[i+1]))
        mod = np.linalg.norm(kinput_scaled[i]-kinput_scaled[i+1])
        directions_scaled.append((kinput_scaled[i]-kinput_scaled[i+1])*num)
        directions.append((kinput_scaled[i]-kinput_scaled[i+1])/mod)
    # =============================================================================
    
    # =============================================================================
    # Remove kpoints already input
    indexes = []
    for i in range(len(allkpoints_scaled)):
        for kpoint in kinput_scaled:
            if(np.array_equal(allkpoints_scaled[i],kpoint)):
                indexes.append(i)
    allkpoints_scaled_no = np.delete(allkpoints_scaled,indexes,axis=0)
    # =============================================================================
    
    # =============================================================================
    # Look for other k points in the same path
    kdef = kinput_real
    kdef_scaled = kinput_scaled
    count = 0
    for index in range(0,len(directions)): #iterating over directions
        count = count+1
        direction = directions[index]
        
        if (all(direction <= np.array([0,0,0]))):
            for kpoint in allkpoints_scaled_no:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
                    kdef = np.insert(kdef,count,kpoint*b,axis=0)
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    count = count+1 
        else:
            for kpoint in allkpoints_scaled_no[::-1]:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
                    kdef = np.insert(kdef,count,kpoint*b,axis=0)
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    count = count+1 
            
    # =============================================================================
    
    # =============================================================================
    #  fake k points only for plotting, projection
    directionsdef_scaled = []
    for i in range(0,len(kdef_scaled)-1):
        directionsdef_scaled.append((kdef_scaled[i]-kdef_scaled[i+1]))
    kk = np.zeros(len(kdef_scaled))
    for i in range(1,len(kk)):
        kk[i] = kk[i-1]+(np.linalg.norm(directionsdef_scaled[i-1]))
    # =============================================================================
           
    return kdef, kdef_scaled, kk


def getk(file_UPOSCAR, N1,N2,N3,kinput_scaled):
    # =============================================================================
    #   Brilloin zone 
    A = np.genfromtxt(file_UPOSCAR,skip_header=2, skip_footer=8)
    to_bohr = 0.5291772108 
    a1,a2,a3 = A[0]*to_bohr/N1,A[1]*to_bohr/N2,A[2]*to_bohr/N3
    c = 2*np.pi
    
    V = np.dot(a1,np.cross(a2,a3))
    b1 = c*np.cross(a2,a3)/V
    b2 = c*np.cross(a3,a1)/V
    b3 = c*np.cross(a1,a2)/V
    h = np.vstack((b1,b2,b3))
    
    b1_mod = np.linalg.norm(b1)
    b2_mod = np.linalg.norm(b2)
    b3_mod = np.linalg.norm(b3)
    b_mod = [b1_mod,b2_mod,b3_mod]
    kinput_real = np.multiply(b_mod,np.array(kinput_scaled))
    #Compute all possible k points
    x1 = np.arange(0,N1+1)/N1
    x2 = np.arange(0,N2+1)/N2
    x3 = np.arange(0,N3+1)/N3
    comb = np.array(np.meshgrid(x1,x2,x3)).T.reshape(-1,3)
    #a1 = np.multiply(comb,b1)
    #a2 = np.multiply(comb,b2)
    #a3 = np.multiply(comb,b3)
    #kpoints = a1+a2+a3
    allkpoints_scaled = comb
    allkpoints = np.multiply(b_mod,comb)
    # =============================================================================
    
    
    # =============================================================================
    # Compute directions <x y z>
    directions_scaled = []
    directions = []
    for i in range(0,len(kinput_scaled)-1):
        if(kinput_scaled[i] not in allkpoints_scaled):
            print('The k point '+str(kinput_scaled[i])+' provided is not compatible with the supercell. \nTry again :)')
        num = 1/np.max(np.abs(kinput_scaled[i]-kinput_scaled[i+1]))
        mod = np.linalg.norm(kinput_scaled[i]-kinput_scaled[i+1])
        directions_scaled.append((kinput_scaled[i]-kinput_scaled[i+1])*num)
        directions.append((kinput_scaled[i]-kinput_scaled[i+1])/mod)
    # =============================================================================
    
    # =============================================================================
    # Remove kpoints already input
    indexes = []
    for i in range(len(allkpoints_scaled)):
        for kpoint in kinput_scaled:
            if(np.array_equal(allkpoints_scaled[i],kpoint)):
                indexes.append(i)
    allkpoints_scaled_no = np.delete(allkpoints_scaled,indexes,axis=0)
    # =============================================================================
    
    # =============================================================================
    # Look for other k points in the same path
    kdef = kinput_real
    kdef_scaled = kinput_scaled
    count = 0
    for index in range(0,len(directions)): #iterating over directions
        count = count+1
        direction = directions[index]
        
        if (all(direction <= np.array([0,0,0]))):
            for kpoint in allkpoints_scaled_no:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
#                    kdef = np.insert(kdef,count,np.dot(h,kpoint),axis=0)   
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    count = count+1 
        else:
            for kpoint in allkpoints_scaled_no[::-1]:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
#                    kdef = np.insert(kdef,count,np.dot(h,kpoint),axis=0)
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    count = count+1 
            
    # =============================================================================
    
    # =============================================================================
    #  fake k points only for plotting, projection
    directionsdef_scaled = []
    for i in range(0,len(kdef_scaled)-1):
        directionsdef_scaled.append((kdef_scaled[i]-kdef_scaled[i+1]))
    kk = np.zeros(len(kdef_scaled))
    for i in range(1,len(kk)):
        kk[i] = kk[i-1]+(np.linalg.norm(directionsdef_scaled[i-1]))
    # =============================================================================
    
    kdef = np.dot(h,kdef_scaled.T).T
    return kdef, kdef_scaled, kk

## =============================================================================
## Parameters
#a = params.get_parameters()[0]
#mba, mti, mo = params.get_parameters()[1:4]
#N1,N2,N3 = params.get_parameters()[4:7]
#kinput = params.get_parameters()[7::][0]
#interp = params.get_parameters()[8]
#savefreq = params.get_parameters()[9]
#
#N1N2N3 = N1*N2*N3 # Number of cells
#N = N1*N2*N3*5    # Number of atoms
## =============================================================================
#a1 = np.array([0,1,1])*a/2
#a2 = np.array([1,0,1])*a/2
#a3 = np.array([1,1,0])*a/2
#kdef, kdef_scaled, kk = getk(a1,a2,a3,N1,N2,N3,kinput)
#kdef_fcc, kdef_fcc_scaled, kk_fcc = getk_fcc(a,N1,N2,N3,kinput)



# -*- coding: utf-8 -*-
"""
Created on Sun Mar 11 11:05:28 2018

@author: Gabriele Coiana
"""

import numpy as np
import yaml, os
#yaml.warnings({'YAMLLoadWarning': False})


def read_parameters(input_file):
    """
    This function takes the input parameters from the file input.txt 
    and applies the right types to the variables
    """
    lista = []
    f = open(input_file, 'r')
    A = f.readlines()
    for string in A:
        index = string.find('=')+2
        lista.append(string[index:-1])
        
    lattice_param = float(lista[1])
    
    m = lista[2]
    Masses = np.fromstring(m, dtype=np.float, sep=',')
    
    ns = lista[3]
    n_atom_unit_cell = np.fromstring(ns, dtype=np.int, sep=',')
    
    ncs = lista[4]
    n_atom_conventional_cell = np.fromstring(ncs, dtype=np.int, sep=',')
    
    N = lista[5]
    n = np.fromstring(N, dtype=np.int, sep=',')
    N1,N2,N3 = n[0],n[1],n[2]
    
    band = lista[6]
    a = np.fromstring(band, dtype=np.float, sep=',')
    num = len(a)/3
    ks = np.split(a,num)
    
    file_eigenvectors = str(lista[7])
    
    file_trajectory = str(lista[8])
    
    file_initial_conf = str(lista[9])
    
    system = str(lista[10])
    
    DT = float(lista[11])
    tau = int(lista[12])
    
    T = float(lista[13])
    
    f.close()
    return lattice_param, Masses, n_atom_unit_cell, n_atom_conventional_cell, N1,N2,N3, ks, file_eigenvectors, file_trajectory, file_initial_conf, system, DT, tau, T




def read_phonopy(file_eigenvectors, n_atom_unit_cell):
    ## =============================================================================
    # Phonopy frequencies and eigenvectors
    data = yaml.load(open(file_eigenvectors))
    #D = data['phonon'][0]['dynamical_matrix']
    #D = np.array(D)
    #D_real, D_imag = D[:,0::2], 1j*D[:,1::2]
    #D = (D_real + D_imag)*21.49068**2#*0.964*10**(4)#
    
    data2 = data['phonon']
    qpoints_scaled = []
    freqs = []
    eigvecs = []
    for element in data2:
        qpoints_scaled.append(element['q-position'])
        freq = []
        eigvec = np.zeros((n_atom_unit_cell*3, n_atom_unit_cell*3),dtype=complex)
        for j in range(len(element['band'])):
            branch = element['band'][j]
            freq.append(branch['frequency'])
            
            eigen = np.array(branch['eigenvector'],dtype=complex)
            eigen_real = eigen[:,:,0]
            eigen_imag = eigen[:,:,1]
            eigen = eigen_real + 1j*eigen_imag
            eigen = eigen.reshape(n_atom_unit_cell*3,)
            eigvec[:,j] = eigen
    
        freqs.append(freq)
        eigvecs.append(eigvec)
    qpoints_scaled = np.array(qpoints_scaled)
    freqs = np.array(freqs)
    Nqpoints = len(qpoints_scaled[:,0])
    
    
    c = 1.88973 #conversion to Bohrs
    Hk = np.array(data['reciprocal_lattice'])*2*np.pi*1/c
    ks = np.dot(Hk,qpoints_scaled.T).T
    # =============================================================================
    return Nqpoints, qpoints_scaled, ks, freqs, eigvecs


def read_SPOSCAR(file,N1N2N3,N,n_atom_conventional_cell,n_atom_primitive_cell):
    with open(file,'r') as f:
        h = f.readlines()[2:5]
    with open('h','w') as g:
        g.writelines(h)
    h = np.loadtxt('h')
    os.remove('h')
    
    S = np.loadtxt(file,skiprows=8)
    
    tot_atoms_primitive = int(np.sum(n_atom_primitive_cell))
    R0 = np.dot(h,S.T).T
    Ruc = np.zeros((tot_atoms_primitive,3))
    for i in range(len(n_atom_conventional_cell)):
        n = n_atom_conventional_cell[i]
        Ruc[i] = R0[i*n*N1N2N3]
    R0 = np.repeat(R0,3,axis=0)
    return Ruc, R0







#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun 20 13:56:19 2019

@author: gabriele
"""
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import animation
import numpy as np
import os

def from_band(ks):
    Points = {'$\Gamma$':[0,0,0],'X':[0.5,0,0],'M':[0.5,0.5,0],'R':[0.5,0.5,0.5],'boh':[0,0.5,0], 'Z':[0,0,0.5]}
    x_labels = []
    for kpoint in ks:
        a = 0
        for element in Points.items():
            a = a + 1
            if (np.array_equal(kpoint , element[1])):
                x_labels.append(element[0])
                break
            if (a==len(Points.items())):
                x_labels.append(' ')
    return x_labels


def animate(frames,plot):
    plot.collections = []
    
#    Baiimag, Tiiimag, Oiimag = frame_BaTiO3[0:8*3], frame_BaTiO3[8*3:9*3], frame_BaTiO3[9*3::]
    plot._offsets3d = (frames[0::3], frames[1::3], frames[2::3])

#    plotBaimag._offsets3d = (Baiimag[0::3],Baiimag[1::3],Baiimag[2::3])
#    plotTiimag._offsets3d = (Tiiimag[0::3],Tiiimag[1::3],Tiiimag[2::3])
#    plotOimag._offsets3d = (Oiimag[0::3],Oiimag[1::3],Oiimag[2::3])
    return

def plot_eigvec(Ruc, n_uc, k,eigvec,freq, ax, fig):
    t = np.arange(0,100,1)
    
    Rucxyz = np.repeat(Ruc,3,axis=0)
    EIGs = eigvec
    
    exp1 = np.exp(1j*np.sum(np.multiply(k,Rucxyz),axis=1)) #this multiplies by the phase factor exp(i k.R)
    #exp2 = np.exp(1j*freq*t).reshape(1,len(t))
    exp2 = np.sin(t).reshape(1,len(t))
    EIG_exp = np.multiply(EIGs,exp1)
    ut = np.dot(EIG_exp.reshape(len(EIGs),1),exp2)
        
    Rt = Ruc.flatten().reshape(len(EIGs),1) + np.real(ut)
    Rt2 = Ruc.flatten().reshape(len(EIGs),1) + np.imag(ut)

    plot = ax.scatter(Rt[0::3,0], Rt[1::3,0], Rt[2::3,0], s=100)
    ani = animation.FuncAnimation(fig,animate,frames=Rt.T,fargs=(plot,))
    return ani

def plot(x,y,title):
    fig,ax = plt.subplots()
    ax.plot(x,y)
    ax.set_xlabel('Frequency [Thz]')
    ax.set_title(title)
    plt.show()
    return

def plot_eigvec_noani(Ruc, n_uc, k,eigvec,freq, ax,fig):
    t = np.arange(0,100,1)
    
    Rucxyz = np.repeat(Ruc,3,axis=0)
    
    eigvec = eigvec.real
    Ruc = Ruc.flatten()
    
    ax.scatter(Ruc[0::3], Ruc[1::3], Ruc[2::3], alpha=0.5, s=100)
    ax.quiver(Ruc[0::3], Ruc[1::3], Ruc[2::3], eigvec[0::3], eigvec[1::3], eigvec[2::3], color='black')
    return 

def plot_with_ani(x,y,ytot, k,eigvec,freq,n,Ruc,eigname,masses):
    fig,ax = plt.subplots()
    graph1, = ax.plot(x,y,label='mode-projected DOS')
    graph2, = ax.plot(x,ytot,'--',label='total DOS')
    ax.set_xlim(left=-1,right=30)
    ax.set_xlabel('Frequency [Thz]')
    ax.set_title('Spectrum of mode n. '+str(n)+' kpoint '+str(k))
    
    ax1 = ax.twiny()
    ax1.set_xlabel('Frequency [cm$^{-1}$]')
    ax1.plot(x*33.35641,-np.ones(len(x)),c='white',linewidth=0,label='')
    ax1.set_xlim(-1*33.35641,30*33.35641)

    
    
    graph3, = ax.plot(np.repeat(freq,100),np.linspace(0,ytot.max(),100),':' ,c='r', label='freq from dispersion')
    plt.legend(handles=[graph1, graph2, graph3], loc=2)
    
    
    ax2 = fig.add_axes([0.5, .5, .3, .3],projection='3d')
    ax2.set_title(str(eigname)+'\nfrequency: '+str(freq))
    ampl = 10
    eigvec = eigvec/np.sqrt(masses)*ampl
    ani = plot_eigvec(Ruc, 1, [0,0,0],eigvec,freq,ax2,fig)
    
    
    print('Mode frequency: ', freq, '\n')
    print('Eigenvector components:')
    print(np.round(np.reshape(np.real(eigvec),np.shape(Ruc)),2))
    print()
    print('Imaginary part:')
    print(np.round(np.reshape(np.imag(eigvec),np.shape(Ruc)),2))
    print()
    
    plt.show()
    return ani

def save_proj(x,y,ytot, k,Ruc, eigvec,freq,n,namedir,masses):
    plt.ioff()
    fig,ax = plt.subplots()
    graph1, = ax.plot(x,y,label='mode-projected DOS')
    graph2, = ax.plot(x,ytot,'--',label='total DOS')
    ax.set_xlabel('Frequency [Thz]')
    ax.set_ylabel('DOS [kB T * ps]')
    fig.suptitle('Spectrum of mode n. '+str(n)+' kpoint '+str(k))
    
    ax1 = ax.twiny()
    ax1.set_xlabel('Frequency [cm$^{-1}$]')
    ax1.plot(x*33.35641,-np.ones(len(x)),c='white',linewidth=0,label='')
    ax1.set_xlim(-1*33.35641,30*33.35641)
    
    
    graph3, = ax.plot(np.repeat(freq,100),np.linspace(0,ytot.max(),100),':' ,c='r', label='freq from dispersion')
    #graph3, = ax.axvline(x=freq,  ymin=0.05, ymax=1, c='blue',linestyle=':', label='freq from dispersion')
#    print('Mode frequency: ', freq, '\n')
#    print('Eigenvector components:')
#    print(np.round(np.reshape(np.real(eigvec),(5,3)),2))
#    print()
#    print('Imaginary part:')
#    print(np.round(np.reshape(np.imag(eigvec),(5,3)),2))
#    print()
#    print()
    name_current_dir = namedir+'/'+str(k)
    try:
        os.mkdir(name_current_dir)
        
    except FileExistsError:
        a = 1
    ax.set_ylim(bottom=-10,top=100)
    ax.set_xlim(left=-1,right=30)
    plt.legend(handles=[graph1, graph2, graph3])
    plt.subplots_adjust(top=0.85)
    
    if(np.allclose(k,[0,0,0])):
        ax2 = fig.add_axes([0.7, 0.4, .2, .2],projection='3d')
        ax2.set_xticks([])
        ax2.set_yticks([])
        ax2.set_zticks([])
        #ax2.xticks([])
    #    ax2.set_title('frequency: '+str(freq))
        ampl = 10
        eigvec = eigvec/np.sqrt(masses)*ampl
        ggraph = plot_eigvec_noani(Ruc, 1 ,[0,0,0],eigvec,freq, ax2, fig)
    
    fig.savefig(name_current_dir+'/proj_'+str(k)+'mode'+str(n)+'.pdf')
    plt.close(fig)
    return 
    
def plot_k(f, data, ks,freqs, title=''):
    """
    Plotting the dispersion from MD and from phonon theory.
    
    Variables:
        f = array of the frequencies [Thz] 
        data = matrix with spectra from all k points: shape = (timesteps,N)
        indexes = list of all kpoints
        freqs = frequencies from phonopy
    """
    fig,ax = plt.subplots()
    plt.suptitle('Spectrum via MD and phonon theory')
    #ax.set_title(title)
    ax.set_ylabel('[Thz]')
    N = len(data[0,:])
    offset = np.zeros(N) + f[0]
    df = f[1]-f[0]
    k = np.linspace(0,1,N)
    dk = k[1] - k[0]
    
    indexes = from_band(ks)

    #norm = [plt.cm.colors.Normalize(vmax=abs(data[0::1,l]).max(), vmin=-abs(data[0::1,l]).min()) for l in range(len(data[0,:]))]
    norm = plt.cm.colors.Normalize(vmax=100, vmin=0)
    
    for j in range(len(data[0::1,0])):
        ax.bar(k, np.repeat(df,N), bottom=offset-df/2,  color=plt.cm.Blues(norm(data[j,:])), tick_label=indexes, align='center', width=dk, alpha=None)
        offset = offset + df
        
    ax2 = plt.twinx()
    ax2.set_ylabel('[Thz]')
    for j in range(14):
        ax2.scatter(k,freqs[:,j], c='b', marker='x', s=100)
    ax2.scatter(k,freqs[:,14], c='b', marker='x', s=100, label='freq from dispersion')
    
        
    ax2.get_shared_y_axes().join(ax, ax2)
    plt.ylim([f[0], f[-1]])
    plt.legend()
    plt.show()
    fig.savefig('spectrum.pdf')
    return
    









































def corr_j(tcorr,X,dt,masses):
    Nsteps = len(tcorr)
    N = np.size(X[0])
    sigma2 = np.var(X,axis=0)
    C = []
    for i in range(Nsteps):
        X_i = X[i::,:]
        Xjj = np.concatenate((X[i::,:],X[0:i,:]))#[i::,:]
        a = np.multiply(np.conjugate(X),Xjj)
        b = 1/(Nsteps) * np.sum(a,axis=0)#/sigma2
        c = np.multiply(b,masses)
        d = 1/N*np.sum(c)
        C.append(d)
    C = np.array(C)
    freq = np.fft.fftfreq(Nsteps,d=dt)
    Z = np.fft.fft(C,axis=0)
    return C, freq, Z

def corr_jaa(tall,X,dt,masses):
    M = len(tall)
    tau = 1000
    tmax = M - tau
    t = np.arange(0,tau)*dt
    N = np.size(X[0])   
    X0 = X[0:tau,:]
    C = []
    for n in range(tau):
        A = []
        for m in range(tmax):
            xj = X[m,:]
            xjj = X[m+n,:]
            a = np.multiply(xj,xjj)
            A.append(a*masses)
        A = np.array(A)
        Cn = np.average(A,axis=0)
        C.append(np.average(Cn))
    C = np.array(C)
    freq = np.fft.fftfreq(tau,d=dt)
    Z = np.fft.fft(C,axis=0)
    return t, C, freq, Z
    
    

def create_folder(system,prefix=''):
    flag = ''
    try:
        namedir = prefix+'phonDOS_'+system
        os.mkdir(namedir)
        flag = 'created'
    except FileExistsError:
        if(flag=='created'):
            bbb = 1
        else:
            number_of_folders = len(np.sort([x[1] for x in os.walk(prefix+'.')][0]))
            print('Folder '+namedir+' already exists. Creating phonDOS_'+str(system)+'_'+str(number_of_folders))
            namedir = namedir+'_'+str(number_of_folders)
            os.mkdir(namedir)
    return namedir
    
    
#this is if you have the velocities instead of positions    
#Vt = np.loadtxt(file_trajectory)[:,1:]*np.sqrt(masses)/np.sqrt(3*N)#/(2.418884254*1e-05)
#Num_timesteps = int(len(Vt[:,0]))
#print(' Number of timesteps of simulation: ', Num_timesteps, '\n')
#tall = np.arange(Num_timesteps)*DT*2.418884254*1e-05 #conversion to picoseconds
#dt_ps = tall[1]-tall[0] 

#eigvec_exp = np.array([0,0,0,0,0,0.5,0,0,-0.9,0,0,-0.6,0,0,0.6])
#a = np.array([0,0,1,0,0,-1,0,0,-1,0,0,-1,0,0,-1])  

#    print('\t\t sum of projected: ',1/2*np.sum(C_proj[0].real)*cH, ' Hartree')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 11 16:55:42 2019

@author: Gabriele Coiana
"""

import numpy as np
from decomp import cell, bz, plot, read

def corr(tall,X,tau,mode):
    M = len(tall)
    dt = tall[1] - tall[0]
    tmax = M - tau
    N = np.size(X[0])   
    X0 = X[0:tmax,:]
    X2 = 1/tmax*np.sum(X[0:tmax,:]*X[0:tmax,:])
    C = []
    for n in range(tau):
        Xjj = X[n:n+tmax,:]
        a = np.multiply(np.conjugate(X0),Xjj)
        b = 1/(tmax) * np.sum(a,axis=0)#/X2
        c = np.multiply(b,1)
        if (mode=='projected'):
            d = c
        else:
            d = np.sum(c)
        C.append(d)
    C = np.array(C)
    t = np.arange(0,tau)*dt
    freq = np.fft.fftfreq(tau,d=dt)
    Z = np.fft.fft(C,axis=0)
    return t, C, freq, Z


# =============================================================================
# Parameters
input_file = '../input'#sys.argv[1]
a = read.read_parameters(input_file)[0]
Masses = read.read_parameters(input_file)[1]
n_atom_unit_cell = read.read_parameters(input_file)[2]
n_atom_conventional_cell = read.read_parameters(input_file)[3]
N1,N2,N3 = read.read_parameters(input_file)[4:7]
kinput = read.read_parameters(input_file)[7::][0]
file_eigenvectors = read.read_parameters(input_file)[8]
file_trajectory = read.read_parameters(input_file)[9]
file_initial_conf = read.read_parameters(input_file)[10]
system = read.read_parameters(input_file)[11]
DT = read.read_parameters(input_file)[12]
tau = read.read_parameters(input_file)[13]
T = read.read_parameters(input_file)[14]


tot_atoms_uc = int(np.sum(n_atom_unit_cell)) 
tot_atoms_conventional_cell = int(np.sum(n_atom_conventional_cell)) 
N1N2N3 = N1*N2*N3 # Number of cells
N = N1*N2*N3*tot_atoms_uc    # Number of atoms
masses, masses_for_animation = util.repeat_masses(Masses, n_atom_conventional_cell, n_atom_unit_cell, N1, N2, N3)
cH = 1.066*1e-6 # to [H]
cev = 2.902*1e-05 # to [ev]
kbH = 3.1668085639379003*1e-06# a.u. [H/K]
kbev = 8.617333262*1e-05 # [ev/K]
# =============================================================================

print('\nHello, lets start!\n')
print(' Getting input parameters and calculating velocities...')
print(' Temperature: ', T, ' K')
Nqpoints, qpoints_scaled, ks, freqs, eigvecs = read.read_phonopy(file_eigenvectors, tot_atoms_uc)
Ruc, R0 = read.read_SPOSCAR(file_initial_conf, N1N2N3, N)                       #rhombo or avg R0



Rt = np.loadtxt(file_trajectory)[:,1:]
Num_timesteps = int(len(Rt[:,0]))
print(' Number of timesteps of simulation: ', Num_timesteps, '\n')
tall = np.arange(Num_timesteps)*DT*2.418884254*1e-05 #conversion to picoseconds
dt_ps = tall[1]-tall[0]
meta = int(Num_timesteps/2)
Vt = np.diff(Rt,axis=0)/dt_ps*np.sqrt(masses)/np.sqrt(3*(N))

#you want the max frequency plotted be 25 Thz
max_freq = 0.5*1/dt_ps
if (max_freq < 25):
    meta = int(tau/2)
else:
    meta = int(tau/2*25/max_freq)
#Vt = np.gradient(Rt,dt_ps,axis=0)*np.sqrt(masses)/np.sqrt(3*(N))  
Vt = np.diff(Rt,axis=0)/dt_ps*np.sqrt(masses)/np.sqrt(3*(N))



## =============================================================================
## Decomposition
Zs = np.zeros((meta,Nqpoints))
print('Now performing decomposition... ')
for i in range(Nqpoints):
    eigvec = eigvecs[i]
    k = ks[i]
    freq_disp = freqs[i]
    print('\tkpoint:  ',k)
    
    #Creating the collective variable based on the k point
    Vcoll = np.zeros((Num_timesteps-1,tot_atoms_conventional_cell*3),dtype=complex)  
    for j,h,l in zip(range(tot_atoms_conventional_cell*3),np.repeat(range(0,N),3)*N1N2N3*3,np.tile(range(0,3),tot_atoms_conventional_cell)):
        vels = np.array(Vt[:,h+l:h+N1N2N3*3:3],dtype=complex)
        poss = R0[h:h+N1N2N3*3:3,:]
        x = np.multiply(vels,np.exp(1j*np.dot(poss,k)))
        Vcoll[:,j] = np.sum(x,axis=1)
        
        
    Tkt = np.zeros((Num_timesteps-1,tot_atoms_uc*3), dtype=complex)
    for l,m in zip(range(0,tot_atoms_conventional_cell*3, every_tot*3),range(0,tot_atoms_uc*3,3)):
        Tkt[:,m:m+3] = Vcoll[:,l:l+3]
    eigvecH = np.conjugate(eigvec.T)
    Qkt = np.dot(eigvecH,Tkt.T).T#.reshape(Num_timesteps,1)
    
    
    t, C, freq, G = corr(tall,Tkt,tau, ' ')
    Ztot = np.sqrt(np.conjugate(G)*G).real*cH/(kbH*T)
    
    print('\t\t kinetic energy of this kpoint: ',1/2*C.real[0]*cH, ' Hartree')
    print('\t\t kinetic according to eqp thm: ',1/2*kbH*T, ' Hartree')
    print('\t\t ratio: ', np.round((1/2*C.real[0]*cH)/(1/2*kbH*T)*100,2), ' %\n')
    
    Zs[:,i] = Ztot[0:meta]
## =============================================================================

np.savetxt('Zs_'+system,Zs)
np.savetxt('freq',freq[0:meta])
plot.plot_k(freq[0:meta],Zs,qpoints_scaled,freqs,title=file_eigenvectors)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 25 12:00:17 2018

@author: Gabriele Coiana
"""
import numpy as np


class BaTiO3:
    def __init__(self,a):
        self.a = a
        self.mBa = 250331.80714328878 #[atomic units]
        self.mTi = 87256.20316855246 #[atomic units]
        self.mO = 29164.392890585812#[atomic units]
        return
        
    def phaseI(self):
        #self.Ti = np.array(np.meshgrid(np.arange(0.5,0.5+self.N1*self.a,self.a), np.arange(0.5,0.5+self.N2*self.a,self.a), np.arange(0.5,0.5+self.N3*self.a,self.a)))
        self.Ti = self.a*np.array([[0.5,0.5,0.5]])
        self.O = self.a*np.array([[0.5,0.5,0], [0.5,0,0.5], [0,0.5,0.5]])
        self.Ba = self.a*np.array([[0,0,0]])
        return
    
    def supercell(self, N1, N2, N3):
        self.N1, self.N2, self.N3 = N1, N2, N3
        xx,yy,zz = np.meshgrid(np.arange(0.5*self.a,0.5+self.N1*self.a,self.a), np.arange(0.5*self.a,0.5+self.N2*self.a,self.a), np.arange(0.5*self.a,0.5+self.N3*self.a,self.a))
        xx, yy, zz = xx.reshape(1,N1*N2*N3)[0], yy.reshape(1,N1*N2*N3)[0], zz.reshape(1,N1*N2*N3)[0]
        self.Ti = np.dstack((zz,xx,yy))[0]
        
        xx,yy,zz = np.meshgrid(np.arange(0.0,0.0+self.N1*self.a,self.a), np.arange(0.0,0.0+self.N2*self.a,self.a), np.arange(0.0,0.0+self.N3*self.a,self.a))
        xx, yy, zz = xx.reshape(1,N1*N2*N3)[0], yy.reshape(1,N1*N2*N3)[0], zz.reshape(1,N1*N2*N3)[0]
        self.Ba = np.dstack((zz,xx,yy))[0]
        
        xx,yy,zz = np.meshgrid(np.arange(0.5*self.a,0.5+self.N1*self.a,self.a), np.arange(0.5*self.a,0.5+self.N2*self.a,self.a), np.arange(0.0,0.0+self.N3*self.a,self.a))
        xx, yy, zz = xx.reshape(1,N1*N2*N3)[0], yy.reshape(1,N1*N2*N3)[0], zz.reshape(1,N1*N2*N3)[0]
        self.O3 = np.dstack((zz,xx,yy))[0]
        
        xx,yy,zz = np.meshgrid(np.arange(0.5*self.a,0.5+self.N1*self.a,self.a), np.arange(0.0,0.0+self.N2*self.a,self.a), np.arange(0.5*self.a,0.5+self.N3*self.a,self.a))
        xx, yy, zz = xx.reshape(1,N1*N2*N3)[0], yy.reshape(1,N1*N2*N3)[0], zz.reshape(1,N1*N2*N3)[0]
        self.O1 = np.dstack((zz,xx,yy))[0]
        
        xx,yy,zz = np.meshgrid(np.arange(0.0,0.0+self.N1*self.a,self.a), np.arange(0.5*self.a,0.5+self.N2*self.a,self.a), np.arange(0.5*self.a,0.5+self.N3*self.a,self.a))
        xx, yy, zz = xx.reshape(1,N1*N2*N3)[0], yy.reshape(1,N1*N2*N3)[0], zz.reshape(1,N1*N2*N3)[0]
        self.O2 = np.dstack((zz,xx,yy))[0]
        return 
    
    def get_supercell(self, a, N1, N2, N3):
        bati = BaTiO3(a)
        bati.supercell(N1,N2,N3)
        Rba,Rti,Ro1,Ro2,Ro3 = bati.Ba,bati.Ti,bati.O1,bati.O2,bati.O3
        R = np.vstack((Rba,Rti,Ro1,Ro2,Ro3))
        R = np.repeat(R,3,axis=0)
        return R
        



## =============================================================================
## Parameters
#a = params.get_parameters()[0]
#mba, mti, mo = params.get_parameters()[1:4]
#N1,N2,N3 = params.get_parameters()[4:7]
#kinput = params.get_parameters()[7::][0]
#interp = params.get_parameters()[8]
#savefreq = params.get_parameters()[9]
#
#N1N2N3 = N1*N2*N3 # Number of cells
#N = N1*N2*N3*5    # Number of atoms
## =============================================================================
#bati = BaTiO3(4)
#bati.supercell(2,2,2)
#print(bati.O3)



#Kr, a, epsilon, sigma, m = 15.18, 7.551345593761988, 0.00485678, 4.5, 1451.0
#const = (Kr, a, epsilon, sigma)
#R0 = np.concatenate((bati.Ba, bati.Ti, bati.O1, bati.O2, bati.O3))
#dt, nSteps, T0, gamma, N = 1, 20, 300, 0.0, 5*8

#Rtot, Vtot, Utot, Ktot, Etot, R2tot  = moleculardynamics.langevin(R0, dt, nSteps, T0, m, gamma, N, *const)

#plot.plot(bati)
